# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/fewsServices/UploadFewsPy.py $
# Last committed:     $Revision: 208453 $; $Author: svcdms $; $Date: 2019-07-09 11:59:27 +0200 (di, 09 jul 2019) $
#
from contextlib import redirect_stdout
from requests import exceptions
from zeep import exceptions as zeep_exceptions
import hkvfewspy
import io


class UploadFewsPy:
    def __init__(self, logger, url, filter_id):
        self._pi = hkvfewspy.Pi()

        try:
            self._pi.setClient(wsdl=url)
            filters = self._pi.getFilters()
            self._filter_id = filters[filter_id]["id"]
        except exceptions.ConnectionError as e:
            logger.error("UploadFewsPy.init: {}".format(str(e)))
            exit(1)
        except KeyError as e:
            logger.error("UploadFewsPy.init: Filter {} not found".format(filters[filter_id]["id"]))
            exit(1)
        else:
            try:
                self._locations = self._pi.getLocations(filterId=self._filter_id)
                self._parameters = self._pi.getParameters(filterId=self._filter_id)
            except zeep_exceptions.Fault as e:
                logger.error("UploadFewsPy.init: {}".format(str(e)))
                exit(1)

    def write_single_dataframe(self, logger, data_reader, data, par_map, module_instance_id, loc):
        success = True
        logger.debug("write_single_dataframe - start")
        for par in par_map:
            logger.debug("writing {}".format(par))
            # set some variable settings
            try:
                location_id = self._locations[loc].locationId
            except AttributeError as e:
                logger.warning("UploadFewsPy.write_series: location id not found in filter ({})".format(str(e)))
            else:
                try:
                    parameter_id = self._parameters[par_map[par]].id
                except (AttributeError, KeyError) as e:
                    logger.warning("UploadFewsPy.write_series: parameter id not found in filter ({})".format(str(e)))
                    logger.warning("UploadFewsPy.write_series: parameters in filter {}".format(list(self._parameters)))
                    logger.warning("UploadFewsPy.write_series: remember to replace dots by _")
                else:
                    # create pi-timeseries object
                    pi_ts = self._pi.setPiTimeSeries()

                    f = io.StringIO()
                    with redirect_stdout(f):
                        # set a header object
                        pi_ts.write.header.moduleInstanceId(module_instance_id)
                        pi_ts.write.header.locationId(location_id)
                        pi_ts.write.header.parameterId(parameter_id)
                        pi_ts.write.header.missVal(-9999)
                        # pi_ts.write.header.ensembleId(eps_id)

                        # set an events object (pandas.Series or pandas.DataFrame)
                        series = data[par]
                        series.name = "value"
                        pi_ts.write.events(series)

                        try:
                            logger.debug("start writing pi-xml")
                            xml = pi_ts.to.pi_xml()
                            # ingest pi-timeseries into fews
                            logger.debug("start soap put")
                            self._pi.putTimeSeriesForFilter(
                                filterId=self._filter_id,
                                piTimeSeriesXmlContent=xml,
                            )
                            logger.debug("ready soap put")
                        except ValueError:
                            logger.error("UploadFewsPy.write_series: problem uploading {},{},{}".format(data_reader.get_fn(), loc, data[par]))
                            success = False

                    logger.debug(f.getvalue())
        logger.debug("write_single_dataframe - finished")
        return success

    def write_series(self, logger, data_reader, module_instance_id, loc):
        logger.debug("UploadFewsPy.write_series: uploading data from file {}".format(data_reader.get_fn()))

        # get the number of data frames
        ndf = data_reader.get_number_of_dfs()

        data = data_reader.get_data()
        par_map = data_reader.get_par_map()
        success = self.write_single_dataframe(logger, data_reader, data, par_map, module_instance_id, loc)

        if ndf == 2:
            # only two supported
            data = data_reader.get_data2()
            par_map = data_reader.get_par_map2()
            success2 = self.write_single_dataframe(logger, data_reader, data, par_map, module_instance_id, loc)
            return success and success2
        else:
            return success
